[loggers]
keys = root, kafka, plugins

[handlers]
keys = console

[formatters]
keys = generic

[logger_root]
level = {{ LOG_LEVEL }}
handlers = console

[logger_kafka]
qualname = monasca_common.confluent_kafka
level = {{ LOG_LEVEL_KAFKA }}
handlers = console
propagate = 0

[logger_plugins]
qualname = monasca_notification.plugins
level = {{ LOG_LEVEL_PLUGINS }}
handlers = console
propagate = 0

[handler_console]
class = logging.StreamHandler
args = (sys.stderr,)
level = DEBUG
formatter = generic

[formatter_generic]
class = oslo_log.formatters.ContextFormatter
